from abc import abstractmethod 
import requests
from datetime import datetime
import traceback

class AbstractBot:
    @abstractmethod
    def send_message(self):
        pass

class AbstractNotifier:
    """Slackbot that automatically captures messages if started or began.
    """
    def __enter__(self):
        self.send_message("Began.")
        return self
    
    def __exit__(self, exc_type, exc_value, traceback):
        if exc_type is None:
            self.send_message("Finished successfully.")
        else:
            if hasattr(traceback, 'print_last'):
                self.send_message(str(exc_type) + str(exc_value) + str(traceback.print_last()))
            else:
                self.send_message(str(exc_type) + str(exc_value))

    def get_current_time(self) -> str:
        return str((datetime.now().time().hour + 10) % 24) + ':' + datetime.now().time().minute.__str__()
        


import requests
import json
from .base import *

def send_message(string, penny=False,embeds=[]):
    obj = {'username': "NEWS", "content": string,"embeds":embeds}
    result = requests.post(penny_discord_url, data=json.dumps(obj), headers={"Content-Type": "application/json"})
    print(result.status_code)

class DiscordBot:
    """Class wrapper for a chatbot in Keybase to work.
    """
    def __init__(self, project_name: str, webhook_url: str, username: str):
        self.project_name = project_name
        self.webhook_url = webhook_url
        self.username = username

    def send_message(self, string: str, penny: bool=False, embeds: list=[]):
        """
            Send messages to Discord Bot.
        """
        obj = {'username': self.username, "content": string,"embeds":embeds}
        response = requests.post(self.webhook_url, data=json.dumps(obj), headers={"Content-Type": "application/json"})

class DiscordNotifier(AbstractNotifier, DiscordBot):
    """
    DIscordBot that automatically messages if started or began.
    """
    def __init__(self, project_name: str, webhook_url: str, username: str):
        super(DiscordNotifier, self).__init__(project_name, webhook_url, username)



"""KeyBase Notifier. A small difference between Keybase and the other notifiers is that Keybase has poorer User interface. 
This means you may want to include more details.
"""
from .base import *

class KeyBaseBot:
    """Class wrapper for a chatbot in Keybase to work.
    """
    def __init__(self, project_name: str, username: str, webhook_url: str):
        self.project_name = project_name
        self.webhook_url = webhook_url

    def send_message(self, message: str, include_time: bool=True):
        """Send Keybase message"""
        if include_time:
            current_time = self.get_current_time()
            response = requests.post(self.webhook_url, data=f"{current_time}: {self.project_name} | " + message)
        else:
            response = requests.post(self.webhook_url, data=f"{self.project_name} | " + message)
        
        if response.status_code != 200:
            raise ValueError(
                'Request to keybase returned an error %s, the response is:\n%s'
                % (response.status_code, response.text)
            )

class KeyBaseNotifier(KeyBaseBot, AbstractNotifier):
    """
    KeyBaseNotifier that automatically messages if started or began.
    """
    def __init__(self, project_name: str, username: str, webhook_url: str):
        super(KeyBaseNotifier, self).__init__(project_name, username, webhook_url)



"""Contextmanager for Slackbot notifications
"""
import requests
import json
from contextlib import AbstractContextManager
from .base import *

def send_message(MESSAGE:str, webhook_url: str, username: str):
    """Upload slack data"""
    slack_data = {'text': username + ": " + MESSAGE}
    response = requests.post(
        webhook_url, data=json.dumps(slack_data),
        headers={'Content-Type': 'application/json'}
    )
    if response.status_code != 200:
        raise ValueError(
            'Request to slack returned an error %s, the response is:\n%s'
            % (response.status_code, response.text)
        )

class SlackBot(AbstractBot):
    """Base SlackBot
    """
    def __init__(self, project_name: str, username: str, webhook_url: str):
        self.project_name = project_name
        self.username = username
        self.webhook_url = webhook_url
    
    def send_message(self, message):
        """Send slack message"""
        slack_data = {'text': self.username + " | " + self.project_name + ": " + message}
        response = requests.post(
            self.webhook_url, data=json.dumps(slack_data),
            headers={'Content-Type': 'application/json'}
        )
        if response.status_code != 200:
            raise ValueError(
                'Request to slack returned an error %s, the response is:\n%s'
                % (response.status_code, response.text)
            )

class SlackNotifier(AbstractNotifier, SlackBot):
    """Slackbot that automatically captures messages if started or began.
    """
    def __init__(self, project_name, username, webhook_url):
        super(SlackNotifier, self).__init__(project_name, username, webhook_url)
    